@@ -6,7 +6,7 @@ use warnings;
 
 use base qw/DBIx::Class::Row/;
 
-=head1 NAME 
+=head1 NAME
 
 DBIx::Class::InflateColumn - Automatically create objects from column data
 
@@ -25,13 +25,13 @@ the column data. It also "deflates" objects into an appropriate format
 for the database.
 
 It can be used, for example, to automatically convert to and from
-L<DateTime> objects for your date and time fields. 
+L<DateTime> objects for your date and time fields.
 
 =head1 METHODS
 
 =head2 inflate_column
 
-Instruct L<DBIx::Class> to inflate the given column. 
+Instruct L<DBIx::Class> to inflate the given column.
 
 In addition to the column name, you must provide C<inflate> and
 C<deflate> methods. The C<inflate> method is called when you access
@@ -50,6 +50,11 @@ corresponding table class using something like:
 (Replace L<DateTime::Format::Pg> with the appropriate module for your
 database, or consider L<DateTime::Format::DBI>.)
 
+The coderefs you set for inflate and deflate are called with two parameters,
+the first is the value of the column to be inflated/deflated, the second is the
+row object itself. Thus you can call C<< ->result_source->schema->storage->dbh >> on
+it, to feed to L<DateTime::Format::DBI>.
+
 In this example, calls to an event's C<insert_time> accessor return a
 L<DateTime> object. This L<DateTime> object is later "deflated" when
 used in the database layer.
@@ -58,8 +63,10 @@ used in the database layer.
 
 sub inflate_column {
   my ($self, $col, $attrs) = @_;
-  $self->throw_exception("No such column $col to inflate") unless $self->has_column($col);
-  $self->throw_exception("inflate_column needs attr hashref") unless ref $attrs eq 'HASH';
+  $self->throw_exception("No such column $col to inflate")
+    unless $self->has_column($col);
+  $self->throw_exception("inflate_column needs attr hashref")
+    unless ref $attrs eq 'HASH';
   $self->column_info($col)->{_inflate_info} = $attrs;
   $self->mk_group_accessors('inflated_column' => $col);
   return 1;
@@ -68,7 +75,8 @@ sub inflate_column {
 sub _inflated_column {
   my ($self, $col, $value) = @_;
   return $value unless defined $value; # NULL is NULL is NULL
-  my $info = $self->column_info($col) || $self->throw_exception("No column info for $col");
+  my $info = $self->column_info($col)
+    or $self->throw_exception("No column info for $col");
   return $value unless exists $info->{_inflate_info};
   my $inflate = $info->{_inflate_info}{inflate};
   $self->throw_exception("No inflator for $col") unless defined $inflate;
@@ -78,17 +86,29 @@ sub _inflated_column {
 sub _deflated_column {
   my ($self, $col, $value) = @_;
   return $value unless ref $value; # If it's not an object, don't touch it
-  my $info = $self->column_info($col) || $self->throw_exception("No column info for $col");
+  my $info = $self->column_info($col) or
+    $self->throw_exception("No column info for $col");
   return $value unless exists $info->{_inflate_info};
   my $deflate = $info->{_inflate_info}{deflate};
   $self->throw_exception("No deflator for $col") unless defined $deflate;
   return $deflate->($value, $self);
 }
 
+=head2 get_inflated_column
+
+  my $val = $obj->get_inflated_column($col);
+
+Fetch a column value in its inflated state.  This is directly
+analogous to L<DBIx::Class::Row/get_column> in that it only fetches a
+column already retreived from the database, and then inflates it.
+Throws an exception if the column requested is not an inflated column.
+
+=cut
+
 sub get_inflated_column {
   my ($self, $col) = @_;
-  $self->throw_exception("$col is not an inflated column") unless
-    exists $self->column_info($col)->{_inflate_info};
+  $self->throw_exception("$col is not an inflated column")
+    unless exists $self->column_info($col)->{_inflate_info};
 
   return $self->{_inflated_column}{$col}
     if exists $self->{_inflated_column}{$col};
@@ -96,12 +116,31 @@ sub get_inflated_column {
            $self->_inflated_column($col, $self->get_column($col));
 }
 
+=head2 set_inflated_column
+
+  my $copy = $obj->set_inflated_column($col => $val);
+
+Sets a column value from an inflated value.  This is directly
+analogous to L<DBIx::Class::Row/set_column>.
+
+=cut
+
 sub set_inflated_column {
   my ($self, $col, @rest) = @_;
   my $ret = $self->_inflated_column_op('set', $col, @rest);
   return $ret;
 }
 
+=head2 store_inflated_column
+
+  my $copy = $obj->store_inflated_column($col => $val);
+
+Sets a column value from an inflated value without marking the column
+as dirty.  This is directly analogous to
+L<DBIx::Class::Row/store_column>.
+
+=cut
+
 sub store_inflated_column {
   my ($self, $col, @rest) = @_;
   my $ret = $self->_inflated_column_op('store', $col, @rest);
@@ -124,6 +163,13 @@ sub _inflated_column_op {
   return $obj;
 }
 
+=head2 update
+
+Updates a row in the same way as L<DBIx::Class::Row/update>, handling
+inflation and deflation of columns appropriately.
+
+=cut
+
 sub update {
   my ($class, $attrs, @rest) = @_;
   $attrs ||= {};
@@ -137,6 +183,13 @@ sub update {
   return $class->next::method($attrs, @rest);
 }
 
+=head2 new
+
+Creates a row in the same way as L<DBIx::Class::Row/new>, handling
+inflation and deflation of columns appropriately.
+
+=cut
+
 sub new {
   my ($class, $attrs, @rest) = @_;
   $attrs ||= {};
